/*
 * arch/ppc/kernel/m8240_pci.c
 * 
 * PCI setup routines for the Motorola SPS Sandpoint Test Platform
 *
 * Author: Mark A. Greer
 *         mgreer@mvista.com
 *
 * Copyright 2000, 2001 MontaVista Software Inc.
 *
 * This program is free software; you can redistribute  it and/or modify it
 * under  the terms of  the GNU General  Public License as published by the
 * Free Software Foundation;  either version 2 of the  License, or (at your
 * option) any later version.
 */
#include <linux/kernel.h>
#include <linux/init.h>
#include <linux/pci.h>
#include <linux/malloc.h>

#include <asm/byteorder.h>
#include <asm/io.h>
#include <asm/irq.h>
#include <asm/uaccess.h>
#include <asm/machdep.h>
#include <asm/pci-bridge.h>

#include "pci.h"
#include "pci_auto.h"
#include "mpc10x.h"

#ifdef CONFIG_CU824
void __init
cu824_pcibios_fixup(void)
{
        struct pci_dev *dev;

	pci_for_each_dev(dev)
	{
		if (dev->devfn == 0x78)
		{
			dev->irq = 1;
			pcibios_write_config_byte(dev->bus->number, dev->devfn,
			                          PCI_INTERRUPT_LINE, dev->irq);
		}
	}
}
#endif

void __init
m8240_find_bridges(void)
{
	struct pci_controller	*hose;

	hose = pcibios_alloc_controller();

	if (!hose)
		return;

	hose->first_busno = 0;
	hose->last_busno = 0xff;

	if (mpc10x_bridge_init(hose,
			       MPC10X_MEM_MAP_B,
			       MPC10X_MEM_MAP_B,
			       MPC10X_MAPB_EUMB_BASE) == 0) {

		hose->mem_resources[0].end = 0xffffffff;
	
		hose->last_busno = pciauto_bus_scan(hose, hose->first_busno);

#if defined (CONFIG_CU824)
		ppc_md.pcibios_fixup = cu824_pcibios_fixup;
		ppc_md.pcibios_fixup_bus = NULL;
#else
		ppc_md.pcibios_fixup = NULL;
		ppc_md.pcibios_fixup_bus = NULL;
#endif

	}
	else {
		if (ppc_md.progress)
			ppc_md.progress("Bridge init failed", 0x100);
		printk("Host bridge init failed\n");
	}

	return;
}
