/*
 * $Id: m8240_setup.c,v 1.1.1.1 2001/11/21 00:36:13 hg Exp $
 *
 *  linux/arch/ppc/kernel/setup.c
 *
 *  Copyright (C) 1995  Linus Torvalds
 *  Adapted from 'alpha' version by Gary Thomas
 *  Modified by Cort Dougan (cort@cs.nmt.edu)
 *  Modified for MBX using prep/chrp/pmac functions by Dan (dmalek@jlc.net)
 *  Further modified for generic 8xx and 8260 by Dan.
 */

/*
 * bootup setup stuff..
 */

#include <linux/config.h>
#include <linux/errno.h>
#include <linux/sched.h>
#include <linux/kernel.h>
#include <linux/mm.h>
#include <linux/stddef.h>
#include <linux/unistd.h>
#include <linux/ptrace.h>
#include <linux/slab.h>
#include <linux/user.h>
#include <linux/a.out.h>
#include <linux/tty.h>
#include <linux/major.h>
#include <linux/interrupt.h>
#include <linux/reboot.h>
#include <linux/init.h>
#include <linux/blk.h>
#include <linux/ioport.h>
#include <linux/irq.h>

#include <asm/mmu.h>
#include <asm/processor.h>
#include <asm/residual.h>
#include <asm/io.h>
#include <asm/pgtable.h>
#include <asm/mpc8240.h>
#include <asm/machdep.h>

#include <asm/time.h>

#include "open_pic.h"

extern void __init
m8240_find_bridges(void);

static u_char cu824_openpic_initsenses[] = {
	1,
	1,
	1,
	1,
	1,
	1,
};


static int m8240_set_rtc_time(unsigned long time)
{
	return 0;
}
static unsigned long m8240_get_rtc_time(void)
{
	return 0;
}
void m8240_calibrate_decr(void);

extern unsigned long loops_per_jiffy;

unsigned char __res[sizeof(bd_t)];
unsigned long empty_zero_page[1024];

extern char saved_command_line[256];

extern unsigned long find_available_memory(void);

void __init adbdev_init(void)
{
}

void __init
m8240_setup_arch(void)
{
	m8240_find_bridges();
}

void
abort(void)
{
#ifdef CONFIG_XMON
	extern void xmon(void *);
	xmon(0);
#endif
	machine_restart(NULL);
	for(;;);
}

/* The decrementer counts at the system (internal) clock frequency
 * divided by four.
 */
void __init m8240_calibrate_decr(void)
{
	bd_t	*binfo = (bd_t *)__res;
	int freq, divisor;

	freq = (binfo->bi_busfreq * 1000000);
        divisor = 4;
        tb_ticks_per_jiffy = freq / HZ / divisor;
	tb_to_us = mulhwu_scale_factor(freq / divisor, 1000000);
}

void
m8240_restart(char *cmd)
{
	extern void m8260_gorom(bd_t *bi, uint addr);
	uint	startaddr;

	/* Most boot roms have a warmstart as the second instruction
	 * of the reset vector.  If that doesn't work for you, change this
	 * or the reboot program to send a proper address.
	 */
	startaddr = 0xfff00104;

	if (cmd != NULL) {
		if (!strncmp(cmd, "startaddr=", 10))
			startaddr = simple_strtoul(&cmd[10], NULL, 0);
	}

	m8260_gorom((void*)__pa(__res), startaddr);
}

void
m8240_power_off(void)
{
   m8240_restart(NULL);
}

void
m8240_halt(void)
{
   m8240_restart(NULL);
}


int m8240_setup_residual(char *buffer)
{
        int     len = 0;
	bd_t	*bp;

	bp = (bd_t *)__res;
			
	len += sprintf(len+buffer,"core clock\t: %ld MHz\n"
		       "bus  clock\t: %ld MHz\n",
		       bp->bi_intfreq /*/ 1000000*/,
		       bp->bi_busfreq /*/ 1000000*/);

	return len;
}

/* Initialize the internal interrupt controller.  The number of
 * interrupts supported can vary with the processor type, and the
 * 8260 family can have up to 64.
 * External interrupts can be either edge or level triggered, and
 * need to be initialized by the appropriate driver.
 */
void __init
m8240_init_IRQ(void)
{
	OpenPIC_InitSenses = cu824_openpic_initsenses;
	OpenPIC_NumInitSenses = sizeof( cu824_openpic_initsenses );
	openpic_init(1, 0, 0, -1);

		/* Enable PCI interrupts within the on-board controller.
		 */
	*(volatile char *)(0xfe80000b) = 0x0f;
}

/*
 * Same hack as 8xx
 */
unsigned long __init m8240_find_end_of_memory(void)
{
	bd_t	*binfo;
	extern unsigned char __res[];
	
	binfo = (bd_t *)__res;

	return binfo->bi_memsize;
}

void __init
m8240_init(unsigned long r3, unsigned long r4, unsigned long r5,
	 unsigned long r6, unsigned long r7)
{

	if ( r3 )
		memcpy( (void *)__res,(void *)(r3+KERNELBASE), sizeof(bd_t) );
	
#ifdef CONFIG_BLK_DEV_INITRD
	/* take care of initrd if we have one */
	if ( r4 )
	{
		initrd_start = r4 + KERNELBASE;
		initrd_end = r5 + KERNELBASE;
	}
#endif /* CONFIG_BLK_DEV_INITRD */
	/* take care of cmd line */
	if ( r6 )
	{
		
		*(char *)(r7+KERNELBASE) = 0;
		strcpy(cmd_line, (char *)(r6+KERNELBASE));
	}

	ppc_md.setup_arch     = m8240_setup_arch;
	ppc_md.setup_residual = m8240_setup_residual;
	ppc_md.get_cpuinfo    = NULL;
	ppc_md.irq_cannonicalize = NULL;
	ppc_md.init_IRQ       = m8240_init_IRQ;
	ppc_md.get_irq	      = openpic_get_irq;
	ppc_md.init           = NULL;

	ppc_md.restart        = m8240_restart;
	ppc_md.power_off      = m8240_power_off;
	ppc_md.halt           = m8240_halt;

	ppc_md.time_init      = NULL;
	ppc_md.set_rtc_time   = m8240_set_rtc_time;
	ppc_md.get_rtc_time   = m8240_get_rtc_time;
	ppc_md.calibrate_decr = m8240_calibrate_decr;

	ppc_md.find_end_of_memory = m8240_find_end_of_memory;

	ppc_md.kbd_setkeycode    = NULL;
	ppc_md.kbd_getkeycode    = NULL;
	ppc_md.kbd_translate     = NULL;
	ppc_md.kbd_unexpected_up = NULL;
	ppc_md.kbd_leds          = NULL;
	ppc_md.kbd_init_hw       = NULL;
#ifdef CONFIG_MAGIC_SYSRQ
	ppc_md.kbd_sysrq_xlate	 = NULL;
#endif

}

/*
 * Copied from prom.c so I don't have include all of that crap.
 *				-- Dan
 *
 * prom_init() is called very early on, before the kernel text
 * and data have been mapped to KERNELBASE.  At this point the code
 * is running at whatever address it has been loaded at, so
 * references to extern and static variables must be relocated
 * explicitly.  The procedure reloc_offset() returns the address
 * we're currently running at minus the address we were linked at.
 * (Note that strings count as static variables.)
 */
extern unsigned long reloc_offset(void);
#define PTRRELOC(x)	((typeof(x))((unsigned long)(x) + offset))

__init
unsigned long
prom_init(uint r3, uint r4, uint r5, uint r6)
{
	unsigned long offset = reloc_offset();
 	unsigned long phys;
	extern char __bss_start, _end;

	/* First zero the BSS -- use memset, some arches don't have
	 * caches on yet */
	memset_io(PTRRELOC(&__bss_start),0 , &_end - &__bss_start);

 	/* Default */
 	phys = offset + KERNELBASE;

	/* We are done.
	*/
	return phys;
}

