/*
 * arch/ppc/kernel/pcippc2_pic.c
 * 
 * Interrupt controller support for Gespac PCIPPC boards.
 *
 * (C) 2002 DENX Software Engineering, Wolfgang Denk, wd@denx.de
 *
 * Based on the K2 board support from HHL.
 *
 * Author: Matt Porter <mporter@mvista.com>
 *
 * Copyright 2000 MontaVista Software Inc.
 *
 * This program is free software; you can redistribute  it and/or modify it
 * under  the terms of  the GNU General  Public License as published by the
 * Free Software Foundation;  either version 2 of the  License, or (at your
 * option) any later version.
 */

#include <linux/stddef.h>
#include <linux/init.h>
#include <linux/threads.h>
#include <linux/spinlock.h>
#include <asm/system.h>
#include <asm/io.h>
#include <linux/interrupt.h>
#include <asm/pcippc2.h>

#include "local_irq.h"
#include "cpc710.h"

static void pcippc2_mask_irq(unsigned int irq);
static void pcippc2_unmask_irq(unsigned int irq);
static void pcippc2_mask_and_ack_irq(unsigned int irq);

struct hw_interrupt_type pcippc2_pic = {
	"PCIPPC-2 PIC",
	NULL,
	NULL,
	pcippc2_unmask_irq,
	pcippc2_mask_irq,
	pcippc2_mask_and_ack_irq,
	NULL,
	NULL
};

static void
pcippc2_unmask_irq(unsigned int irq)
{
	if ((irq >= 0) && (irq < 32)) {
		/* Know IRQ fits in entry 0 of ppc_cached_irq_mask[] */
		ppc_cached_irq_mask[0] &= ~(1 << irq);

		outl(ppc_cached_irq_mask[0], FPGA(INT,INTR_MASK));
	}
	return;
}

static void
pcippc2_mask_irq(unsigned int irq)
{
	if ((irq >= 0) && (irq < 32)) {
		/* Know IRQ fits in entry 0 of ppc_cached_irq_mask[] */
		ppc_cached_irq_mask[0] |= (1 << irq);

		outl(ppc_cached_irq_mask[0], FPGA(INT,INTR_MASK));
		inl(FPGA(INT,INTR_MASK));
	}
	return;
}

static void
pcippc2_mask_and_ack_irq(unsigned int irq)
{
	if ((irq >= 0) && (irq < 32)) {
		/* Know IRQ fits in entry 0 of ppc_cached_irq_mask[] */
		ppc_cached_irq_mask[0] |= (1 << irq);

		outl(ppc_cached_irq_mask[0], FPGA(INT,INTR_MASK));
		outb(irq, FPGA(INT,INTR_EOI));
	}
	return;
}

__init void
pcippc2_init_IRQ(void)
{
	int i;

	ppc_cached_irq_mask[0] = 0xffffffff;

	outl(0xffffffff, FPGA(INT,INTR_MASK));
	outl(0, FPGA(INT,INTR_VECTOR_BASE));
	outb(0, FPGA(INT,INTR_TYPE));

	for ( i = 0 ; i < 32 ; i++ ) {
		irq_desc[i].status = IRQ_DISABLED | IRQ_LEVEL;
		irq_desc[i].handler = &pcippc2_pic;
	}
}

int
pcippc2_get_irq(struct pt_regs *regs)
{
	int irq;

	irq = inl(PCIPPC_PCI32_BAR + INTACK);

	if ((irq >= 0) && (irq < 32)) {
		ppc_irq_dispatch_handler( regs, irq );
		return irq;
	}

	printk("Bogus interrupt from PC = %lx\n", regs->nip);
	ppc_spurious_interrupts++;
	return -1;
}
