/*
 * arch/ppc/kernel/pcippc2_setup.c
 * 
 * Board setup routines for Gespac PCIPPC boards.
 *
 * (C) 2002 DENX Software Engineering, Wolfgang Denk, wd@denx.de
 *
 * Based on the K2 board support from HHL.
 *
 * Author: Matt Porter <mporter@mvista.com>
 *
 * Copyright 2001 MontaVista Software Inc.
 *
 * This program is free software; you can redistribute  it and/or modify it
 * under  the terms of  the GNU General  Public License as published by the
 * Free Software Foundation;  either version 2 of the  License, or (at your
 * option) any later version.
 */

#include <linux/config.h>
#include <linux/stddef.h>
#include <linux/kernel.h>
#include <linux/init.h>
#include <linux/errno.h> 
#include <linux/reboot.h>
#include <linux/pci.h>
#include <linux/kdev_t.h>
#include <linux/types.h>
#include <linux/major.h>
#include <linux/blk.h>
#include <linux/console.h>
#include <linux/delay.h>
#include <linux/ide.h>

#include <asm/system.h>
#include <asm/pgtable.h>
#include <asm/page.h>
#include <asm/dma.h>
#include <asm/io.h>
#include <asm/machdep.h>
#include <asm/mk48t59.h>
#include <asm/time.h>
#include <asm/pcippc2.h>
#include <linux/serial.h>
#include <asm/pcippc2_serial.h>

#include "cpc710.h"

extern void pcippc2_init_IRQ(void);
extern int pcippc2_get_irq(struct pt_regs *);
extern void pcippc2_find_bridges(void);
extern unsigned long loops_per_jiffy;

extern unsigned long reloc_offset(void);
extern void init_doc_config(unsigned long addr);

static unsigned int pcippc2_l2_cache_size;

unsigned long pcippc2_fpga0_phys;
unsigned long pcippc2_fpga1_phys;

u_char todc_direct_read_val(int addr);
void todc_direct_write_val(int addr, unsigned char val);
int mk48t59_set_rtc_time(unsigned long nowtime);
unsigned long mk48t59_get_rtc_time(void);
static long __init mk48t59_init(void);
void __init pcippc2_calibrate_decr(void);

unsigned char __res[sizeof(bd_t)];

static unsigned int cpu_7xx[16] = {
	0, 15, 14, 0, 0, 13, 5, 9, 6, 11, 8, 10, 16, 12, 7, 0
};
static unsigned int cpu_6xx[16] = {
	0, 0, 14, 0, 0, 13, 5, 9, 6, 11, 8, 10, 0, 12, 7, 0
};

static inline unsigned long _get_HID1(void)
{
	unsigned long val;

	__asm__ __volatile__("mfspr %0,1009" : "=r" (val));
	return val;
}                                                                               

int
pcippc2_get_bus_speed(void)
{
	int bus_speed;

	switch (inl(FPGA(BOARD,ID)) & PCIPPC_BUS_FREQ_MASK)
	{
		case PCIPPC_BUS_FREQ_100MHZ:
			bus_speed = 100000000;
			break;

		default:
			printk("Unknown bus frequency, Assuming 100 Mhz.\n");
			bus_speed = 100000000;
			break;
	}

	return bus_speed;
}

int
pcippc2_get_cpu_speed(void)
{
	unsigned long hid1;
	int cpu_speed;

	hid1 = _get_HID1() >> 28;

	if ((_get_PVR()>>16) == 8)
		hid1 = cpu_7xx[hid1];
	else
		hid1 = cpu_6xx[hid1];

	cpu_speed = pcippc2_get_bus_speed()*hid1/2;
	return cpu_speed;
}

void __init pcippc2_calibrate_decr(void)
{
	int freq, divisor = 4;

	/* determine processor bus speed */
	freq = pcippc2_get_bus_speed();
	tb_ticks_per_jiffy = freq / HZ / divisor;
	tb_to_us = mulhwu_scale_factor(freq/divisor, 1000000);
}

void
pcippc2_setup_l2_cache(void)
{
  unsigned int l2clkdiv;

	switch (inl(FPGA(BOARD,ID)) & PCIPPC_L2CACHE_MASK)
	{
		case PCIPPC_L2CACHE_512KB:
			pcippc2_l2_cache_size = L2CR_L2SIZ_512KB;
			break;

		case PCIPPC_L2CACHE_1MB:
			pcippc2_l2_cache_size = L2CR_L2SIZ_1MB;
			break;

		default:
		case PCIPPC_L2CACHE_NONE:
			pcippc2_l2_cache_size = 0;
			break;
	}

	switch (inl(FPGA(BOARD,ID)) & PCIPPC_L2DIV_MASK)
	{
		case PCIPPC_L2DIV_2_5:
			l2clkdiv = L2CR_L2CLK_DIV2_5;
			break;
		case PCIPPC_L2DIV_2:
			l2clkdiv = L2CR_L2CLK_DIV2;
			break;
		case PCIPPC_L2DIV_1_5:
			l2clkdiv = L2CR_L2CLK_DIV1_5;
			break;
		case PCIPPC_L2DIV_3:
			l2clkdiv = L2CR_L2CLK_DIV3;
			break;
	}


	if (pcippc2_l2_cache_size)
	{
		_set_L2CR(0);
		_set_L2CR(L2CR_L2E | L2CR_L2PE       | pcippc2_l2_cache_size |
			        l2clkdiv | L2CR_L2RAM_PIPE | L2CR_L2OH_1_0);
	}
}

void
pcippc2_get_l2_string(char *buffer)
{
	switch (pcippc2_l2_cache_size)
	{
		case L2CR_L2SIZ_512KB:
			sprintf(buffer, "512KB");
			break;

		case L2CR_L2SIZ_1MB:
			sprintf(buffer, "1MB");
			break;

		default:
			sprintf(buffer, "0KB");
			break;
	}
}

int
pcippc2_get_26(void)
{
  if( inl(FPGA(BOARD,ID)) & PCIPPC_26_MASK)
    return 6;
	else
		return 2;
}

int
pcippc2_get_system(void)
{
  if( inl(FPGA(BOARD,ID)) & PCIPPC_SYSTEM_MASK)
    return 1;
	else
		return 0;
}

int
pcippc2_get_idstring(char * buffer)
{
	int len=0;

  /* Generate string for exact board reference */
  len = sprintf( buffer, "PCIPPC-");

  /* Add 2 or 6 */
  if(pcippc2_get_26() == 6)
		len += sprintf( buffer+len, "6");
	else
  	len += sprintf( buffer+len, "2");

  /* Add 1 letter for the CPU frequency */
  switch (inl(FPGA(BOARD,ID)) & PCIPPC_BUS_FREQ_MASK)
	{
		case PCIPPC_BUS_FREQ_100MHZ: /* 100 MHz bus */

			switch (_get_HID1() >> 28)
      {
      	case 0:
        case 3:
        case 4:
        case 6:
        case 15: len += sprintf(buffer+len, "?"); /* reserved */
                 break;
        case 1:  len += sprintf(buffer+len, "W"); /* 750 MHz */
                 break;
        case 2:  len += sprintf(buffer+len, "U"); /* 700 MHz */
                 break;
        case 5:  len += sprintf(buffer+len, "S"); /* 650 MHz */
                 break;
        case 7:  len += sprintf(buffer+len, "K"); /* 450 MHz */
                 break;
        case 8:  len += sprintf(buffer+len, "E"); /* 300 MHz */
                 break;
        case 9:  len += sprintf(buffer+len, "O"); /* 550 MHz */
                 break;
        case 10: len += sprintf(buffer+len, "I"); /* 400 MHz */
                 break;
        case 11: len += sprintf(buffer+len, "M"); /* 500 MHz */
                 break;
        case 12: len += sprintf(buffer+len, "Y"); /* 800 MHz */
                 break;
        case 13: len += sprintf(buffer+len, "Q"); /* 600 MHz */
                 break;
        case 14: len += sprintf(buffer+len, "G"); /* 350 MHz */
                 break;

      } /* end switch */
 			break;

		default: /* Unknown bus speed */
                 len += sprintf(buffer+len, "?");
			break;
	}

  /* add 1 letter for the L2 cache Size: */
  switch (inl(FPGA(BOARD,ID)) & PCIPPC_L2CACHE_MASK)
  {
		case PCIPPC_L2CACHE_1MB:    len += sprintf(buffer+len, "1"); /* Cache Size 1MBytes */
																 break;

		case PCIPPC_L2CACHE_512KB:  len += sprintf(buffer+len, "5"); /* Cache Size 512Kbytes */
																 break;

		case PCIPPC_L2CACHE_NONE:   len += sprintf(buffer+len, "0"); /* No L2 cache */
																 break;

		default:                     len += sprintf(buffer+len, "?"); /* Disable L2 Cache : Size Reserved ! */
																 break;
	}

  /* add 1 letter for the DRAM size */
  if (*((unsigned long*) RGBAN1) < 64L*1024L*1024L)
		len += sprintf(buffer+len, "?");
	else if (*((unsigned long*) RGBAN1) < 128L*1024L*1024L)
		len += sprintf(buffer+len, "A");
	else if (*((unsigned long*) RGBAN1) < 256L*1024L*1024L)
		len += sprintf(buffer+len, "B");
	else if (*((unsigned long*) RGBAN1) < 512L*1024L*1024L)
		len += sprintf(buffer+len, "C");
	else if (*((unsigned long*) RGBAN1) < 1024L*1024L*1024L)
		len += sprintf(buffer+len, "D");
	else
		len += sprintf(buffer+len, "E");

  return len;
}


int
pcippc2_get_cpuinfo(char *buffer)
{
	int len;
	unsigned char pcippc2_geo_bits;
	char l2_string[10];

	len = sprintf( buffer, "vendor\t\t: GESPAC\n");

  len += sprintf( buffer+len, "machine\t\t: ");

  len += pcippc2_get_idstring(buffer+len);

	len += sprintf( buffer+len, "\n");


	len += sprintf( buffer+len, "cpu speed\t: %dMhz\n",
			pcippc2_get_cpu_speed()/1000000);

	len += sprintf( buffer+len, "bus speed\t: %dMhz\n",
			pcippc2_get_bus_speed()/1000000);

	pcippc2_get_l2_string(l2_string);
	len += sprintf( buffer+len, "L2\t\t: %s\n", l2_string);

	len += sprintf( buffer+len, "memory type\t: SDRAM\n"); 

	pcippc2_geo_bits = (inl(FPGA(BOARD,ID)) & PCIPPC_GEO_ADR_MASK) >> PCIPPC_GEO_ADR_SHIFT;

  if( pcippc2_get_system() )
  {
      len += sprintf( buffer+len,
		  "backplane\t: System slot board with geographical address %x\n",
		  pcippc2_geo_bits);
  }
	else
  {
      len += sprintf( buffer+len,
		  "backplane\t: Peripheral slot board with geographical address %x\n",
		  pcippc2_geo_bits);
  }

	len += sprintf(buffer+len, "\n");

	return len;
}


static void __init
pcippc2_fpga_init(void) {
	unsigned long addr;

	outl((HW_FPGA_FUNCNO | PCI_BASE_ADDRESS_0 | 0x80000000), 
	     PCIPPC_PCI32_CONFIG_ADDR);
	addr = (inl(PCIPPC_PCI32_CONFIG_DATA) & ~0x3) 
	  - PCIPPC_PCI32_LOWER_IO + PCIPPC_PCI32_IO_BASE;
	pcippc2_fpga0_phys = (unsigned long) ioremap(addr, 0x8000);

	outl((HW_FPGA_FUNCNO | PCI_BASE_ADDRESS_1 | 0x80000000), 
	     PCIPPC_PCI32_CONFIG_ADDR);
	addr = (inl(PCIPPC_PCI32_CONFIG_DATA) & ~0xf) 
	  - PCIPPC_PCI32_LOWER_MEM + PCIPPC_PCI32_MEM_BASE;
	pcippc2_fpga1_phys = (unsigned long) ioremap(addr, 0x8000);
}

static void __init 
pcippc2_init_serial(void) {
	struct serial_struct req;

	/*
	 * We do early_serial_setup() to clean out the rs-table[] from the
	 * statically compiled in version.
	 */
	memset(&req, 0, sizeof(struct serial_struct));
	req.baud_base = BASE_BAUD;
	req.flags = (ASYNC_BOOT_AUTOCONF | ASYNC_SKIP_TEST);
	req.io_type = SERIAL_IO_MEM;
	req.iomem_reg_shift = 2;

	req.line = 0;
	req.iomem_base = (void *)(pcippc2_fpga0_phys + HW_FPGA0_UART1);
	req.irq = COM1_INT;
	early_serial_setup(&req);

	req.line = 1;
	req.iomem_base = (void *)(pcippc2_fpga0_phys + HW_FPGA0_UART2);
	req.irq = COM2_INT;
	early_serial_setup(&req);
}

extern char cmd_line[];

void __init
pcippc2_setup_arch(void)
{
	unsigned int cpu;
  char idstr[15];

	pcippc2_fpga_init();

	/* Configure and enable L2 cache */
	pcippc2_setup_l2_cache();

	/* init to some ~sane value until calibrate_delay() runs */
        loops_per_jiffy = 50000000/HZ;

	/* Setup PCI host bridges */
	pcippc2_find_bridges();

	pcippc2_init_serial();

#if defined(CONFIG_MTD_DOC1000) || defined(CONFIG_MTD_DOC2000)
	init_doc_config(pcippc2_fpga1_phys + HW_FPGA1_DOC);
#endif

#ifdef CONFIG_BLK_DEV_INITRD
	if (initrd_start)
		ROOT_DEV = MKDEV(RAMDISK_MAJOR, 0); /* /dev/ram */
	else
#endif
#ifdef CONFIG_ROOT_NFS
		ROOT_DEV = to_kdev_t(0x00FF);	/* /dev/nfs pseudo device */
#else
		ROOT_DEV = to_kdev_t(0x1601);	/* /dev/hdc1 */
#endif

	/* Identify the system */
	printk("System Identification: ");

	idstr[pcippc2_get_idstring(&idstr[0])]=0;
  printk(idstr);

  printk(" - PowerPC 750 @ %d Mhz\n", pcippc2_get_cpu_speed()/1000000);

	/* Identify the CPU manufacturer */
	cpu = _get_PVR();
	printk("CPU manufacturer: %s [rev=%04x]\n", (cpu & (1<<15)) ? "IBM" :
	       "Motorola", (cpu & 0xffff));
}

void
pcippc2_restart(char *cmd)
{
	outl(0, SPOR);
	barrier();

	for(;;);
}

void
pcippc2_power_off(void)
{
	for(;;);
}

void
pcippc2_halt(void)
{
	pcippc2_restart(NULL);
}

/*
 * Set BAT 3 to map 0xff000000.
 */
static __inline__ void
pcippc2_set_bat(void)
{
	unsigned long   bat3u, bat3l;
	static int	mapping_set = 0;

	if (!mapping_set)
	{
		__asm__ __volatile__(
				" lis %0,0xff00\n \
				ori %1,%0,0x002a\n \
				ori %0,%0,0x01fe\n \
				mtspr 0x21e,%0\n \
				mtspr 0x21f,%1\n \
				isync\n \
				sync "
				:: "r" (bat3u), "r" (bat3l));
		mapping_set = 1;
	}
	return;
}

unsigned long __init pcippc2_find_end_of_memory(void)
{
	pcippc2_set_bat();

	return *((unsigned long*) RGBAN1);
}

void __init pcippc2_init(unsigned long r3, unsigned long r4, unsigned long r5,
			 unsigned long r6, unsigned long r7)
{
	if ( r3 )
		memcpy( (void *)__res,(void *)(r3+KERNELBASE), sizeof(bd_t) );
	
#ifdef CONFIG_BLK_DEV_INITRD
	if ( r4 )
	{
		initrd_start = r4 + KERNELBASE;
		initrd_end = r5 + KERNELBASE;
	}
#endif

	/* Copy cmd_line parameters */
	if ( r6 )
	{
		*(char *)(r7 + KERNELBASE) = 0;
		strcpy(cmd_line, (char *)(r6 + KERNELBASE));
	} 

	pci_dram_offset = PCIPPC_PCI32_SYS_MEM_BASE;

	ppc_md.setup_arch = pcippc2_setup_arch;
	ppc_md.setup_residual = NULL;
	ppc_md.get_cpuinfo = pcippc2_get_cpuinfo;
	ppc_md.irq_cannonicalize = NULL;
	ppc_md.init_IRQ = pcippc2_init_IRQ;
	ppc_md.get_irq = pcippc2_get_irq;
	ppc_md.init = NULL;

	ppc_md.find_end_of_memory = pcippc2_find_end_of_memory;
	
	ppc_md.restart = pcippc2_restart;
	ppc_md.power_off = pcippc2_power_off;
	ppc_md.halt = pcippc2_halt;

	ppc_md.time_init = mk48t59_init;
	ppc_md.set_rtc_time = mk48t59_set_rtc_time;
	ppc_md.get_rtc_time = mk48t59_get_rtc_time;
	ppc_md.calibrate_decr = pcippc2_calibrate_decr;

	ppc_md.nvram_read_val = todc_direct_read_val;
	ppc_md.nvram_write_val = todc_direct_write_val;
}

#define PTRRELOC(x)	((typeof(x))((unsigned long)(x) + offset))

unsigned long
prom_init(void)
{
	unsigned long offset = reloc_offset();
	unsigned long phys;
	extern char __bss_start, _end;

	/* First zero the BSS -- use memset, some arches don't have
	 * caches on yet */
	memset_io(PTRRELOC(&__bss_start),0 , &_end - &__bss_start);

 	/* Default */
 	phys = offset + KERNELBASE;

	return phys;
}

/* Access routines when RTC accessed directly (like normal memory) */
u_char
todc_direct_read_val(int addr)
{
	return inb(pcippc2_fpga1_phys + HW_FPGA1_RTC + addr);
}
  
void
todc_direct_write_val(int addr, unsigned char val)
{
	outb(val, pcippc2_fpga1_phys + HW_FPGA1_RTC + addr);
	return;
}

int mk48t59_set_rtc_time(unsigned long nowtime)
{
	unsigned char save_control;
	struct rtc_time tm;


	to_tm(nowtime, &tm);

	/* tell the clock it's being written */
	save_control = ppc_md.nvram_read_val(MK48T59_RTC_CONTROLA);
	
	ppc_md.nvram_write_val(MK48T59_RTC_CONTROLA,
			     (save_control | MK48T59_RTC_CA_WRITE));

        tm.tm_year = (tm.tm_year - 1900) % 100;
	BIN_TO_BCD(tm.tm_sec);
	BIN_TO_BCD(tm.tm_min);
	BIN_TO_BCD(tm.tm_hour);
	BIN_TO_BCD(tm.tm_mon);
	BIN_TO_BCD(tm.tm_mday);
	BIN_TO_BCD(tm.tm_year);

	ppc_md.nvram_write_val(MK48T59_RTC_SECONDS,      tm.tm_sec);
	ppc_md.nvram_write_val(MK48T59_RTC_MINUTES,      tm.tm_min);
	ppc_md.nvram_write_val(MK48T59_RTC_HOURS,        tm.tm_hour);
	ppc_md.nvram_write_val(MK48T59_RTC_MONTH,        tm.tm_mon);
	ppc_md.nvram_write_val(MK48T59_RTC_DAY_OF_MONTH, tm.tm_mday);
	ppc_md.nvram_write_val(MK48T59_RTC_YEAR,         tm.tm_year);
	
	/* Turn off the write bit. */
	ppc_md.nvram_write_val(MK48T59_RTC_CONTROLA, save_control);

	return 0;
}

unsigned long mk48t59_get_rtc_time(void)
{
	unsigned char save_control;
	unsigned int year, mon, day, hour, min, sec;

	/* Simple: freeze the clock, read it and allow updates again */
	save_control = ppc_md.nvram_read_val(MK48T59_RTC_CONTROLA);
	save_control &= ~MK48T59_RTC_CA_READ;
	ppc_md.nvram_write_val(MK48T59_RTC_CONTROLA, save_control);

	/* Set the register to read the value. */
	ppc_md.nvram_write_val(MK48T59_RTC_CONTROLA,
			     (save_control | MK48T59_RTC_CA_READ));

	sec = ppc_md.nvram_read_val(MK48T59_RTC_SECONDS);
	min = ppc_md.nvram_read_val(MK48T59_RTC_MINUTES);
	hour = ppc_md.nvram_read_val(MK48T59_RTC_HOURS);
	day = ppc_md.nvram_read_val(MK48T59_RTC_DAY_OF_MONTH);
	mon = ppc_md.nvram_read_val(MK48T59_RTC_MONTH);
	year = ppc_md.nvram_read_val(MK48T59_RTC_YEAR);

	/* Let the time values change again. */
	ppc_md.nvram_write_val(MK48T59_RTC_CONTROLA, save_control);

	BCD_TO_BIN(sec);
	BCD_TO_BIN(min);
	BCD_TO_BIN(hour);
	BCD_TO_BIN(day);
	BCD_TO_BIN(mon);
	BCD_TO_BIN(year);

	year = year + 1900;
	if (year < 1970) {
		year += 100;
	}

	return mktime(year, mon, day, hour, min, sec);
}

static long __init mk48t59_init(void) {
	unsigned char tmp;

	tmp = ppc_md.nvram_read_val(MK48T59_RTC_CONTROLB);
	if (tmp & MK48T59_RTC_CB_STOP) {
		printk("Warning: RTC was stopped, date will be wrong.\n");
		ppc_md.nvram_write_val(MK48T59_RTC_CONTROLB, 
				       tmp & ~MK48T59_RTC_CB_STOP);
		/* Low frequency crystal oscillators may take a very long
		 * time to startup and stabilize. For now just ignore the
		 * the issue, but attempting to calibrate the decrementer
		 * from the RTC just after this wakeup is likely to be very 
		 * inaccurate. Firmware should not allow to load
		 * the OS with the clock stopped anyway...
		 */
	}
	/* Ensure that the clock registers are updated */
	tmp = ppc_md.nvram_read_val(MK48T59_RTC_CONTROLA);
	tmp &= ~(MK48T59_RTC_CA_READ | MK48T59_RTC_CA_WRITE);
	ppc_md.nvram_write_val(MK48T59_RTC_CONTROLA, tmp);
	return 0;
}

