/*
 * LWMON MPC8xx CPM I2C interface.
 * Copyright (c) 2001 DENX Software Engineering, Wolfgang Denk, wd@denx.de
 *
 * LWMON specific parts of the i2c interface
 */

#include <linux/kernel.h>
#include <linux/ioport.h>
#include <linux/module.h>
#include <linux/init.h>
#include <linux/stddef.h>
#include <linux/parport.h>

#include <asm/mpc8xx.h>
#include "../../arch/ppc/8xx_io/commproc.h"

#include <linux/i2c.h>
#include <linux/i2c-algo-8xx.h>

static void
lwmon_iic_init(struct i2c_algo_8xx_data *data)
{
	volatile cpm8xx_t *cp;
	volatile immap_t *immap;

	cp = cpmp;	/* Get pointer to Communication Processor */
	immap = (immap_t *)IMAP_ADDR;	/* and to internal registers */

	data->iip = (iic_t *)&cp->cp_dparam[PROFF_IIC];

#ifdef USE_IIC_PATCH
	/* Check for and use a microcode relocation patch.
	*/
	if ((data->reloc = data->iip->iic_rpbase))
		data->iip = (iic_t *)&cp->cp_dpmem[data->iip->iic_rpbase];
#endif
		
	data->i2c = (i2c8xx_t *)&(immap->im_i2c);
	data->cp = cp;

	/* Initialize Port B IIC pins.
	*/
	cp->cp_pbpar |= 0x00000030;
	cp->cp_pbdir |= 0x00000030;
	cp->cp_pbodr |= 0x00000030;

	/* Allocate space for two transmit and two receive buffer
	 * descriptors in the DP ram.
	 */
	data->dp_addr = m8xx_cpm_dpalloc(sizeof(cbd_t) * 4);

	/* ptr to i2c area */
	data->i2c = (i2c8xx_t *)&(((immap_t *)IMAP_ADDR)->im_i2c);
}

static int lwmon_install_isr(int irq, void (*func)(void *), void *data)
{
	/* install interrupt handler */
	cpm_install_handler(irq, func, data);

	return 0;
}

static int lwmon_reg(struct i2c_client *client)
{
	return 0;
}

static int lwmon_unreg(struct i2c_client *client)
{
	return 0;
}

static void lwmon_inc_use(struct i2c_adapter *adap)
{
#ifdef MODULE
	MOD_INC_USE_COUNT;
#endif
}

static void lwmon_dec_use(struct i2c_adapter *adap)
{
#ifdef MODULE
	MOD_DEC_USE_COUNT;
#endif
}

static struct i2c_algo_8xx_data lwmon_data = {
	setisr: lwmon_install_isr
};


static struct i2c_adapter lwmon_ops = {
	"lwmon",
	I2C_HW_MPC8XX_LWMON,
	NULL,
	&lwmon_data,
	lwmon_inc_use,
	lwmon_dec_use,
	lwmon_reg,
	lwmon_unreg,
};

int __init i2c_lwmon_init(void)
{
	printk("i2c-lwmon.o: i2c LWMON module\n");

	/* reset hardware to sane state */
	lwmon_iic_init(&lwmon_data);

	if (i2c_8xx_add_bus(&lwmon_ops) < 0) {
		printk("i2c-lwmon: Unable to register with I2C\n");
		return -ENODEV;
	}

	return 0;
}

void __exit i2c_lwmon_exit(void)
{
	i2c_8xx_del_bus(&lwmon_ops);
}

#ifdef MODULE
MODULE_AUTHOR("Wolfgang Denk <wd@denx.de>");
MODULE_DESCRIPTION("I2C-Bus adapter routines for LWMON");

module_init(i2c_lwmon_init);
module_exit(i2c_lwmon_exit);
#endif

