/*
 * arch/ppc/kernel/pcippc2.h
 * 
 * Definitions for Gespac PCIPPC boards support
 *
 * (C) 2002 DENX Software Engineering, Wolfgang Denk, wd@denx.de
 *
 * Based on code from:
 *
 * Author: Matt Porter <mporter@mvista.com>
 *
 * Copyright 2001 MontaVista Software Inc.
 *
 * This program is free software; you can redistribute	it and/or modify it
 * under  the terms of	the GNU General	 Public License as published by the
 * Free Software Foundation;  either version 2 of the  License, or (at your
 * option) any later version.
 */

#ifndef __PPC_KERNEL_PCIPPC2_H
#define __PPC_KERNEL_PCIPPC2_H

#include <linux/config.h>
 
#include <asm/ppcboot.h>

/* The "residual" data board information structure the boot loader
 * hands to us.
 */
extern unsigned char __res[];

/*
 * PCIPPC-2 / PCIPPC-6 definitions
 */

#define PCIPPC_PCI32_BAR		0xff400000	/* Local */
#define PCIPPC_PCI64_BAR		0xff500000	/* cPCI */

#define PCIPPC_PCI32_CONFIG_ADDR	(PCIPPC_PCI32_BAR + 0x000f8000)
#define PCIPPC_PCI32_CONFIG_DATA	(PCIPPC_PCI32_BAR + 0x000f8010)

#define PCIPPC_PCI64_CONFIG_ADDR	(PCIPPC_PCI64_BAR + 0x000f8000)
#define PCIPPC_PCI64_CONFIG_DATA	(PCIPPC_PCI64_BAR + 0x000f8010)

#define PCIPPC_PCI32_SYS_MEM_BASE	0x80000000

#define PCIPPC_PCI32_MEM_BASE		0xf8000000
#define PCIPPC_PCI32_IO_BASE		0xfe000000

#define PCIPPC_PCI64_MEM_BASE		0xf0000000
#define PCIPPC_PCI64_IO_BASE		0xfc000000

#define PCIPPC_PCI32_LOWER_MEM		0x40000000
#define PCIPPC_PCI32_UPPER_MEM		0x43ffffff
#define PCIPPC_PCI32_LOWER_IO		0x04000000
#define PCIPPC_PCI32_UPPER_IO		0x04ffffff

#define PCIPPC_PCI64_LOWER_MEM		0x00000000
#define PCIPPC_PCI64_UPPER_MEM		0x07ffffff
#define PCIPPC_PCI64_LOWER_IO		0x00000000
#define PCIPPC_PCI64_UPPER_IO		0x01ffffff

#define PCIPPC_ISA_IO_BASE		PCIPPC_PCI32_IO_BASE
#define PCIPPC_ISA_MEM_BASE		PCIPPC_PCI32_MEM_BASE

/* The FPGA is a function 0 of device 1 at bus 0.
 */
#define HW_FPGA_FUNCNO			(0 << 8 | 1 << 11 | 0 << 16)

extern unsigned long			pcippc2_fpga0_phys;
extern unsigned long			pcippc2_fpga1_phys;

#define FPGA(r,p)			(pcippc2_fpga0_phys + HW_FPGA0_##r##_##p)

#define HW_FPGA0_INT			0x0000
#define HW_FPGA0_UART1			0x0080
#define HW_FPGA0_UART2			0x0100
#define HW_FPGA0_RTC			0x2000
#define HW_FPGA0_DOC			0x4000
#define HW_FPGA1_RTC			0x0000
#define HW_FPGA1_DOC			0x4000

#define HW_FPGA0_INT_INTR_MASK		0x30
#define HW_FPGA0_INT_INTR_STATUS	0x34
#define HW_FPGA0_INT_INTR_VECTOR_BASE	0x38
#define HW_FPGA0_INT_INTR_TYPE		0x3c
#define HW_FPGA0_INT_INTR_EOI		0x40

#define HW_FPGA0_WDT_CTRL		0x44
#define HW_FPGA0_WDT_PROG		0x48
#define HW_FPGA0_WDT_VAL		0x4c
#define HW_FPGA0_WDT_REFRESH		0x50

#define HW_FPGA0_BOARD_ID		0x60

#define PCIPPC_L2CACHE_MASK		0x00000006	/* Mask for 2 L2 Cache bits */
#define PCIPPC_L2CACHE_512KB		0x00000002	/* 512KB */
#define PCIPPC_L2CACHE_1MB		0x00000000	/* 1MB */
#define PCIPPC_L2CACHE_NONE		0x00000006	/* None */

#define PCIPPC_L2DIV_MASK		0x00600000  /* L2 Cache frequency ratio */
#define PCIPPC_L2DIV_2_5		0x00000000
#define PCIPPC_L2DIV_2			0x00200000
#define PCIPPC_L2DIV_1_5		0x00400000
#define PCIPPC_L2DIV_3			0x00600000

#define PCIPPC_BUS_FREQ_MASK		0x00000180	/* Mask for 2 BUS speed bits */
#define PCIPPC_BUS_FREQ_100MHZ		0x00000000	/* 100MHz */

#define PCIPPC_GEO_ADR_MASK		0x001f0000	/* Mask and shift for */
#define PCIPPC_GEO_ADR_SHIFT		16		/* geographic addressing of cPCI */

#define PCIPPC_SYSTEM_MASK		0x00000200	/* System or peripheral slot */

#define PCIPPC_26_MASK			0x00000400 /* PCIPPC6 or PCIPPC2 */


#define PCIPPC_PMC_DEVNO		6	   /* Should be last device */

/* IRQ levels definition.
 */
#define TIMER0_INT			27
#define TIMER1_INT			26
#define COM1_INT			25
#define COM2_INT			24

#define SCSI_INT1			22
#define SCSI_INT2			21
#define ETHERNET1_INT			20
#define ETHERNET2_INT			19
#define FIREWIRE_INT			18

#define LPCI_INTA			14
#define LPCI_INTB			13
#define LPCI_INTC			12
#define LPCI_INTD			11

#define PCI_INTA			8
#define PCI_INTB			7
#define PCI_INTC			6
#define PCI_INTD			5

#endif /* __PPC_KERNEL_PCIPPC2_H */
