/*
 * (C) Copyright 2000
 * Jrg Haider, SIEMENS AG
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	 See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 * MA 02111-1307 USA
 *
 * (C) 2001, 2002 Wolfgang Denk, wd@denx.de -- Cleanup, Modifications for 2.4 kernels
 * (C) 2001 Wolfgang Denk, wd@denx.de -- Adaption for MAX706TESA Watchdog (LWMON)
 * (C) 2001 Steven Hein,  ssh@sgi.com -- Added timeout configuration option
 */

/*
 * The purpose of this header file is to provide an interface for the
 * driver of the watchdog timer wdt_pcippc2. In essence this interface
 * consists of the three macros WDT_PCIPPC_INIT, WDT_PCIPPC_SERVICE,
 * WDT_PCIPPC_CLOSE, and its functionality is described as follows:
 *
 * WDT_PCIPPC_INIT:    opens the driver and initializes the timer to
 *		       300 seconds;
 *
 * WDT_PCIPPC_SERVICE: writes the value defined by the macro
 *		       WDT_PCIPPC_DEF_SERVICE_TIME to the variable,
 *		       which serves as a timer counter;
 *
 * WDT_PCIPPC_CLOSE:   closes the watchdog driver;
 *
 * Finally there is a macro called WDT_PCIPPC_SET_SERVICE_TIME(sec)
 * for altering the value written to the timer counter to a value,
 * which is specified by sec.
 */


#ifndef _wdt_pcippc2_h
#define _wdt_pcippc2_h

#define WDT_PCIPPC_IOCTL_BASE	'W'

#define WDT_PCIPPC_OPEN_ONLY	_IO(WDT_PCIPPC_IOCTL_BASE, 0)
#define WDT_PCIPPC_ALWAYS	_IO(WDT_PCIPPC_IOCTL_BASE, 1)

#ifndef __KERNEL__

#include <fcntl.h>
#include <unistd.h>
#include <linux/ioctl.h>

#define WDT_PCIPPC_DEVICE "/dev/watchdog"
#define WDT_PCIPPC_DEF_SERVICE_TIME 300

int wdt_pcippc2_fd;
int wdt_pcippc2_value = WDT_PCIPPC_DEF_SERVICE_TIME;

#define WDT_PCIPPC_INIT (wdt_pcippc2_fd = open(WDT_PCIPPC_DEVICE, O_RDWR, 0))

#define WDT_PCIPPC_SET_SERVICE_TIME(sec) wdt_pcippc2_value = (sec);

#define WDT_PCIPPC_SERVICE write(wdt_pcippc2_fd, (char *) &wdt_pcippc2_value, sizeof (wdt_pcippc2_value))

#define WDT_PCIPPC_CLOSE close(wdt_pcippc2_fd)

#endif	/* __KERNEL__ */

#endif	/* _wdt_pcippc2_h */
